/*!
 * vue-router-layout v0.1.2
 * https://github.com/ktsn/vue-router-layout
 *
 * @license
 * Copyright (c) 2018 katashin
 * Released under the MIT license
 * https://github.com/ktsn/vue-router-layout/blob/master/LICENSE
 */
import Vue from 'vue';

/**
 * Find which layout the component should render.
 * It traverses `layout` option from the leaf component to root,
 * and returns the first matched one.
 * If there are no maching, `default` is used.
 */
function findLayoutName(matched) {
    var reversed = matched.slice().reverse();
    var layoutName = 'default';
    for (var _i = 0, reversed_1 = reversed; _i < reversed_1.length; _i++) {
        var record = reversed_1[_i];
        var Component = record.components.default;
        if (Component) {
            var isCtor = typeof Component === 'function' && Component.options;
            var name_1 = isCtor ? Component.options.layout : Component.layout;
            if (name_1) {
                layoutName = name_1;
                break;
            }
        }
    }
    return layoutName;
}
function createRouterLayout(resolve) {
    return Vue.extend({
        name: 'RouterLayout',
        data: function () {
            return {
                layoutName: undefined,
                layouts: Object.create(null)
            };
        },
        watch: {
            layoutName: function (name) {
                if (!this.layouts[name]) {
                    this.$set(this.layouts, name, function () { return resolve(name); });
                }
            }
        },
        beforeRouteEnter: function (to, _from, next) {
            next(function (vm) {
                vm.layoutName = findLayoutName(to.matched);
            });
        },
        beforeRouteUpdate: function (to, _from, next) {
            this.layoutName = findLayoutName(to.matched);
            next();
        },
        render: function (h) {
            var layout = this.layoutName && this.layouts[this.layoutName];
            if (!layout) {
                return h();
            }
            return h(layout, {
                key: this.layoutName
            });
        }
    });
}

export { createRouterLayout };
